import std.file;
import std.string;
import std.process;
import std.stdio;
import std.c.time;

const int width		= 64;
const int height	= 48;
const int span		= 16;

static assert (width % 4 == 0);


char[] splitNbr(char[] nam, int i) {
	int n0 = i % 10;
	int n1 = (i / 10) % 10;
	int n2 = (i / 100) % 10;
	int n3 = (i / 1000);
	return format(" -version=%s0_%s -version=%s1_%s -version=%s2_%s -version=%s3_%s ", nam, n0, nam, n1, nam, n2, nam, n3);
}


align(1) struct BMPHeader {
	ubyte	type1 = 0x42;
	ubyte	type2 = 0x4d;
	uint	bfSize = 0;
	uint	reserved = 0;
	uint	offBits = 0x36;
	uint	size	= 0x28;
	uint	width;
	uint	height;
	ushort	planes = 1;
	ushort	bitCount = 0x18;
	uint	compression = 0;
	uint	sizeImage = 0;
	uint	xppm = 0;
	uint	yppm = 0;
	uint	used = 0;
	uint	imp = 0;
}

static assert (BMPHeader.sizeof == 54);


void main() {
	BMPHeader bmpHdr;
	bmpHdr.width = width;
	bmpHdr.height = height;
	bmpHdr.sizeImage = width * height * 3;
	std.file.write("result.bmp", (cast(void*)&bmpHdr)[0 .. BMPHeader.sizeof]);
	auto startTime = time(null);


	void[] data = std.file.read("result.bmp");

	for (int y = 0; y < height; ++y) {
		for (int x = 0; x < width; x += span) {
			char[] cmd = "dmd -c -o- ctrace.d ctparams.d ";
			cmd ~= splitNbr("w", width);
			cmd ~= splitNbr("h", height);
			cmd ~= splitNbr("x", x);
			cmd ~= splitNbr("y", y);
			cmd ~= splitNbr("s", span);
			cmd ~= " >line.data";
			//writefln(cmd);
			system(cmd);

			//data = std.file.read("result.bmp");

			version (Windows)
				void[] line = std.file.read("line.data")[0..$-2];
			else
				void[] line = std.file.read("line.data")[0..$-1];

			data ~= line;

			static int lastProgress = 0;
			int totalSize = width * height;
			int curSize = x + span + y * width;
			int progress = 100 * curSize / totalSize;
			if (progress > lastProgress) {
				writefln("Done: %s%%", progress);
				lastProgress = progress;
			}
		}

		try {
			std.file.write("result.bmp", data);
		} catch (Object err) {
			writefln("warning: could not save result.bmp");
		}
	}

	auto endTime = time(null);
	writefln("\nImage compiled in %s seconds", endTime - startTime);


	while (true) {
		try {
			std.file.write("result.bmp", data);
		} catch (Object err) {
			writefln("warning: could not save result.bmp");
			continue;
		}

		break;
	}
}
